#!/bin/sh
# TODO:
# - Check if installation is successful
# - Check if already installed
# - Uninstall

# main
set_lang
set_version

if command -v zenity > /dev/null ; then
    echo "DEBUG: Start zenity" # DEBUG
    MENU="zenity"
    zenity --title "$TITLE" --text "$PROMPT_WELCOME" --info
    ! command -v curl > /dev/null && zenity --title "$TITLE" --text "$PROMPT_ERR_DEPS curl" --error && exit 1

    echo "DEBUG: Serial number is required for download" # DEBUG
    SERIAL="$(zenity --title "$TITLE" --text "$PROMPT_SERIAL" --entry)"
    [ -z "$SERIAL" ] && zenity --title "$TITLE" --text "$PROMPT_ERR_SERIAL" --error && exit 1

    
    echo "DEBUG: Generate tempkey & Define DOWNLOAD_URL" # DEBUG
    ARCHIVE="$(get_archive)"
    TEMPKEY="${ARCHIVE##* }"
    FILE="${ARCHIVE%% *}"
    DOWNLOAD_URL="$(printf 'https://soportefirmadigital.com/sfdj/getiso.aspx?tempkey=%s' "$TEMPKEY")"
    [ -z "$DOWNLOAD_URL" ] && zenity --title "$TITLE" --text "$PROMPT_ERR_DOWNLOAD" --error && exit 1
    [ -z "$FILE" ] && zenity --title "$TITLE" --text "$PROMPT_ERR_DOWNLOAD" --error && exit 1
    SAVE_DIR="/tmp/soportefirmadigital"
    SAVE_FILE="$SAVE_DIR/$FILE"
    mkdir -p "$SAVE_DIR"
    SIZE="$(curl -sI "$DOWNLOAD_URL" | sed '/[Cc]ontent-[Ll]ength/!d;s/^.*: //g' | awk '{$1/=1024;printf "%d",$1}')"

    echo "DEBUG: Download file & show progress" # DEBUG
    (curl -sL "$DOWNLOAD_URL" -o "$SAVE_FILE") &
    while true ; do
        sleep 0.5
        DOWN="$(du "$SAVE_FILE" 2>/dev/null | awk '{print $1}')" ; [ -z "$DOWN" ] && DOWN=0
        r=$(((DOWN*10000)/SIZE))
        printf '%d\n' ${r%??}
    done | zenity --title "$TITLE" --text "$PROMPT_DOWNLOAD" --progress --auto-close
    while true ; do
        ACTIVE="$(ps aux | grep 'curl.*soportefirmadigital' | sed '/grep/d')"
        [ -z "$ACTIVE" ] && break
        sleep 0.5
    done | zenity --title "$TITLE" --text "$PROMPT_DOWNLOAD" --progress --pulsate --auto-close
    ACTIVE="$(ps aux | grep 'curl.*soportefirmadigital' | sed '/grep/d')"
    [ -n "$ACTIVE" ] && zenity --title "$TITLE" --text "$PROMPT_ERR_DOWNLOAD" --error && exit 1

    echo "DEBUG: Ask & check sudo password" # DEBUG
    SUDO_PASSWORD="$(zenity --title "$TITLE" --password)"
    [ -z "$SUDO_PASSWORD" ] && zenity --title "$TITLE" --text "$PROMPT_ERR_DEPS_INSTALL" --error && exit 1
    CORRECT_SUDO_PASSWORD="$(printf '%s' "$SUDO_PASSWORD" | sudo -Skp '' whoami >/dev/null 2>&1 || printf 'no')"
    [ "$CORRECT_SUDO_PASSWORD" = "no" ] && zenity --title "$TITLE" --text "$PROMPT_ERR_DEPS_INSTALL" --error && exit 1

    # Attempt to install, forward output to zenity
    # but keep exit code of install function
    echo "DEBUG: Install dependencies, components and certificates according to OS" # DEBUG
    ( ( ( (install_certs; echo $? >&3) |
        zenity --title "$TITLE" --text "$PROMPT_DEPS_INSTALL" --progress --pulsate --auto-close >&4) 3>&1 ) |
        (read -r xs; exit "$xs") ) 4>&1
    [ "$?" != "0" ] && zenity --title "$TITLE" --text "$PROMPT_ERR_DEPS_INSTALL" --error && exit 1

    zenity --title "$TITLE" --text "$PROMPT_END_SUCCESS" --info

else
    MENU="term"
    term_prompt "$TITLE" "$PROMPT_WELCOME" info && read -r NULL
    ! command -v curl && term_prompt "$TITLE" "$PROMPT_ERR_DEPS curl" error && exit 1

    # Serial number is required for download
    term_prompt "$TITLE" "$PROMPT_SERIAL" entry && read -r SERIAL
    [ -z "$SERIAL" ] && term_prompt "$TITLE" "$PROMPT_ERR_SERIAL" error && exit 1

    echo "DEBUG: Generate tempkey & Define DOWNLOAD_URL" # DEBUG
    ARCHIVE="$(get_archive)"
    TEMPKEY="${ARCHIVE##* }"
    FILE="${ARCHIVE%% *}"
    DOWNLOAD_URL="$(printf 'https://soportefirmadigital.com/sfdj/getiso.aspx?tempkey=%s' "$TEMPKEY")"
    [ -z "$DOWNLOAD_URL" ] && term_prompt "$TITLE" "$PROMPT_ERR_DOWNLOAD" error && exit 1
    [ -z "$FILE" ]         && term_prompt "$TITLE" "$PROMPT_ERR_DOWNLOAD" error && exit 1
    SAVE_DIR="/tmp/soportefirmadigital"
    SAVE_FILE="$SAVE_DIR/$FILE"
    mkdir -p "$SAVE_DIR"
    SIZE="$(curl -sI "$DOWNLOAD_URL" | sed '/[Cc]ontent-[Ll]ength/!d;s/^.*: //g' | awk '{$1/=1024;printf "%d",$1}')"

    # Download file & show progress
    term_prompt "$TITLE" "$PROMPT_DOWNLOAD" info && echo
    curl "$DOWNLOAD_URL" -o "$SAVE_FILE" --progress-bar

    # Install dependencies, components and
    # certificates according to OS
    term_prompt "$TITLE" "$PROMPT_DEPS_INSTALL" info && echo
    install_certs "$SAVE_FILE" || term_prompt "$TITLE" "$PROMPT_ERR_DEPS_INSTALL" error && echo

    term_prompt "$TITLE" "$PROMPT_END_SUCCESS" info && echo

fi
